#include "retis_touch.h"

#ifdef __ERIKA__
#include "ee.h"
#else
#error "RETIS_TOUCH: ERIKA OS is required by the motionboard."
#endif

#define STATIC static

STATIC uint8_t touch_read_mode;

void touch_init(uint8_t channel, touch_conf_data_t *init)
{
	switch (channel) {
	case 0 :
		touch_read_mode = init->read_mode;
		touch_set_dimension(TOUCH_X_AXIS, init->x_range); //EE
		touch_set_dimension(TOUCH_Y_AXIS, init->y_range); //EE
		touch_raw_init(); //EE
		
		// Nice hard coding for touch calibration.
		init->tune.xt1 = 2893;
		init->tune.yt1 = 1420;
		init->tune.xt2 = 1864;
		init->tune.yt2 = 2260;
		init->tune.xt3 = 834;
		init->tune.yt3 = 579;
		touch_tune(&init->tune); //EE
			
		//if(init-> autostart) touch_start();
		touch_start();
		break;
	}		
}

void touch_conf(uint8_t channel, uint8_t tag, void *value)
{
	switch(channel) {	
	case 0 :
		switch(tag) {
		case SET_X_RANGE_TAG :
			touch_set_dimension(TOUCH_X_AXIS, *(uint16_t *)value); //EE
			break;
		case SET_Y_RANGE_TAG :
			touch_set_dimension(TOUCH_Y_AXIS, *(uint16_t *)value); //EE
			break;
		case SET_POLL_TAG :
			touch_read_mode &= ~BLOCKING;
			break;			
		case SET_WAIT_TAG :
			touch_read_mode |= BLOCKING;
			break;
		case SET_RAW_TAG :
			touch_read_mode |= RAW_FORMAT;
			break;
		case SET_UNSIGNED_TAG :
			touch_read_mode &= ~RAW_FORMAT;
			touch_read_mode &= ~SIGNED_FORMAT;
			break;
		case SET_SIGNED_TAG :
			touch_read_mode &= ~RAW_FORMAT;
			touch_read_mode |= SIGNED_FORMAT;
			break;
		case SET_READ_MODE_TAG :
			touch_read_mode = 0x07 & (*(uint8_t *)value);
			break;			
		}
	}
}

int8_t touch_read_coord(
		uint8_t channel,
		uint8_t coord,
		void *value)
{
	switch(channel) {
	case 0 :
		if(touch_read_mode & BLOCKING) {
			if(touch_read_mode & RAW_FORMAT)
				touch_wait_raw_position(coord, value);
			else if(touch_read_mode & UNSIGNED_FORMAT)
					touch_wait_u_position(coord, value);
			else touch_wait_s_position(coord, value);
		} else { // POLLING
			if(touch_read_mode & RAW_FORMAT)
				touch_poll_raw_position(coord, value);
			else if(touch_read_mode & SIGNED_FORMAT)
				touch_poll_s_position(coord, value);
			else touch_poll_u_position(coord, value);
		}
		break;
	}
	return 0;
}
