#ifndef __retis_servomotor_h__
#define __retis_servomotor_h__

#include "retis_types.h"

/** 
* @brief \todo  Document this
*
* The \c unit of the angle is given by
* \f[ UNIT = frac{1}{angle\_unit} [degrees] \f]
*/
struct retis_servomotor_conf_data {
	uint16_t angle_unit;	/**< Scale factor for angle unit, typically 1.*/
	int32_t min_angle;	/**< Minimum Angle in angle unit.*/
	int32_t max_angle;	/**< Maximum Angle in angle unit.*/
	uint32_t pwm_period;	/**< Period of the PWM in micro seconds.*/
	uint32_t pw_per_unit;	/**/
	uint32_t pw_at_zero;	/**< Duration of the pulse width in 
				     micro seconds.*/
};

/** 
* @brief Servo Motor Attributes
* 
* This is the list of the valid attribute for the \ref retis_servomotor_config
* function.
* The corresponding expected values are:
* - \ref RETIS_SERVOMOTOR_POWER :
*   \c uint8_t with values 0 = off, 1 = on
*   default is off
*/
enum retis_servomotor_attribute_t {
	RETIS_SERVOMOTOR_POWER,
};

int8_t retis_servomotor_init(uint8_t id, 
			     const struct retis_servomotor_conf_data *info);
int8_t retis_servomotor_conf(uint8_t id, uint8_t attribute, void *value);
int8_t retis_servomotor_write(uint8_t id, int16_t value);
int8_t retis_servomotor_read(uint8_t id, int16_t *value);

#endif /* __retis_servomotor_h__ */
