#include "ee.h"
#include "cpu/pic30/inc/ee_irqstub.h"
#include "retis_servomotor.h"

_FWDT(FWDTEN_OFF);      /* Watchdog Timer Enabled/disabled by user software */
_FGS(GCP_OFF);          /* Disable Code Protection */
_FOSCSEL(FNOSC_PRIPLL); /* Primary (XT, HS, EC) Oscillator without PLL */
_FOSC(OSCIOFNC_ON & POSCMD_XT); /* OSC2 Pin Function: OSC2 is Clock Output
                                 * Primary Oscillator Mode: XT Crystanl */
#define UPDATE_PERIOD 500

/******************************************************************************/
/*                          Function Definitions                              */
/******************************************************************************/
static void timer_set(void)
{
	/* TIMER 1  */
	T1CON = 0;		/* Stops the Timer1 and reset control reg */
	TMR1 = 0;		/* Clear contents of the timer register	*/
	PR1 = 0x9C40;		/* Load the Period register -> 1ms @ 40MHz */
	//PR1 = 0x4E20;		/* Load the Period register -> 500us @ 40MHz */
	//PR1 = 0x2710;		/* Load the Period register -> 250us @ 40MHz */
	IPC0bits.T1IP = 5;	/* Set Timer1 priority to 5 */
	IFS0bits.T1IF = 0;	/* Clear the Timer1 interrupt status flag */
	IEC0bits.T1IE = 1;	/* Enable Timer1 interrupts */
	T1CONbits.TON = 1;	/* Start Timer1 without prescaler */
}
/******************************************************************************/
/*                                  TASKs                                     */
/******************************************************************************/
TASK(TEST)
{
	static EE_INT16 a0 = 0; 
	static EE_INT16 a1 = 0; 

	retis_servomotor_write(0, a0);
	retis_servomotor_write(1, a1);
	a0 = (a0 + 1) % 100;
	a1 = (a1 - 1) % 100;
}

/******************************************************************************/
/*                                   MAIN                                     */
/******************************************************************************/
int main(void)
{
	EE_INT8 retv;
	EE_UINT8 v;
	struct retis_servomotor_conf_data conf;

	/* Clock setup for 40 MHz */
	CLKDIVbits.DOZEN   = 0;
	CLKDIVbits.PLLPRE  = 0;
	CLKDIVbits.PLLPOST = 0;
	PLLFBDbits.PLLDIV  = 78;
	while (OSCCONbits.LOCK != 1) ;

	/* Init basic components */	
	timer_set();		
	EE_leds_init();
	EE_switch_init();
	EE_daughter_leds_init();

	/* Start test*/
	conf.angle_unit = 1;
	conf.min_angle = -90;
	conf.max_angle = 90;
	conf.pwm_period = 20000;
	conf.pw_per_unit = 10;
	conf.pw_at_zero = 1500;
	
	retv = retis_servomotor_init(0, &conf);
	if (retv < 0) {
		EE_leds_on();
		for (;;) ;
	}
	retv = retis_servomotor_init(1, &conf);
	if (retv < 0) {
		EE_leds_on();
		for (;;) ;
	}
	EE_led_on();
	while (!EE_switch_1_get()) ;
	v = 1;
	retis_servomotor_conf(0, RETIS_SERVOMOTOR_POWER, &v);
	if (retv < 0) {
		EE_leds_on();
		for (;;) ;
	}
	retis_servomotor_conf(1, RETIS_SERVOMOTOR_POWER, &v);
	if (retv < 0) {
		EE_leds_on();
		for (;;) ;
	}
	EE_led_off();
	while (!EE_switch_2_get()) ;
	SetRelAlarm(ALARM_TEST, 1000, UPDATE_PERIOD);
	EE_led_on();
	while (!EE_switch_3_get()) ;
	CancelAlarm(ALARM_TEST);
	EE_led_off();
	while (!EE_switch_4_get()) ;
	v = 0;
	retis_servomotor_conf(0, RETIS_SERVOMOTOR_POWER, &v);
	if (retv < 0) {
		EE_leds_on();
		for (;;) ;
	}
	retis_servomotor_conf(1, RETIS_SERVOMOTOR_POWER, &v);
	if (retv < 0) {
		EE_leds_on();
		for (;;) ;
	}
	EE_led_on();

	/* Forever loop: background activities (if any) should go here */
	for (;;) ;

	return 0;
}

/******************************************************************************/
/*                                     ISRs                                   */
/******************************************************************************/
ISR2(_T1Interrupt)
{
	IFS0bits.T1IF = 0;
	CounterTick(TASK_COUNTER);
}
