#include <stdio.h>
#include "string.h"

#include "ee.h"
#include "cpu/pic30/inc/ee_irqstub.h"
#include "retis_touch.h"
#include "console_serial.h"

_FWDT(FWDTEN_OFF);      /* Watchdog Timer Enabled/disabled by user software */
_FGS(GCP_OFF);          /* Disable Code Protection */
_FOSCSEL(FNOSC_PRIPLL); /* Primary (XT, HS, EC) Oscillator without PLL */
_FOSC(OSCIOFNC_ON & POSCMD_XT); /* OSC2 Pin Function: OSC2 is Clock Output
                                 * Primary Oscillator Mode: XT Crystanl */

//#define MDB_RETIS_TOUCH_TEST_RAW
#define MDB_RETIS_TOUCH_TEST_UNSIGNED
//#define MDB_RETIS_TOUCH_TEST_SIGNED

#define MDB_RETIS_TOUCH_CHANNEL 0
#define MDB_RETIS_TOUCH_X_RANGE 240
#define MDB_RETIS_TOUCH_Y_RANGE 180

volatile unsigned char format_index;
touch_conf_data_t tcd;
char str[40];
console_descriptor_t *my_console_1;
#define MY_FIRST_SERIAL 1
#define MY_FIRST_CONSOLE 0

/******************************************************************************/
/*                          Function Definitions                              */
/******************************************************************************/
static void timer_set(void)
{
	/* TIMER 1  */
	T1CON = 0;		/* Stops the Timer1 and reset control reg */
	TMR1 = 0;		/* Clear contents of the timer register	*/
	PR1 = 0x9C40;		/* Load the Period register -> 1ms @ 40MHz */
	IPC0bits.T1IP = 5;	/* Set Timer1 priority to 5 */
	IFS0bits.T1IF = 0;	/* Clear the Timer1 interrupt status flag */
	IEC0bits.T1IE = 1;	/* Enable Timer1 interrupts */
	T1CONbits.TON = 1;	/* Start Timer1 without prescaler */
}
/******************************************************************************/
/*                                  TASKs                                     */
/******************************************************************************/
TASK(TASK_SEND)
{
	EE_INT16 x,y;

	touch_read_coord(MDB_RETIS_TOUCH_CHANNEL,TOUCH_X_AXIS,&x);
	touch_read_coord(MDB_RETIS_TOUCH_CHANNEL,TOUCH_Y_AXIS,&y);

	if(format_index == 0)
		sprintf(str,"x:%5d y:%5d [RAW]\n",x,y);
	else if(format_index == 1)
		sprintf(str,"x:%5d y:%5d [UNSIGNED]\n",x,y);
	else if(format_index == 2)
		sprintf(str,"x:%5d y:%5d [SIGNED]\n",x,y);
	else if(format_index == 3)
		sprintf(str,"x:%5d y:%5d [RAW]\n",x,y);
	else if(format_index == 4)
		sprintf(str,"x:%5d y:%5d [UNSIGNED]\n",x,y);
	else if(format_index == 5)
		sprintf(str,"x:%5d y:%5d [SIGNED]\n",x,y);

	console_write(MY_FIRST_CONSOLE, str, strlen(str));

	if(EE_button_get_S1()) {
		format_index = (++format_index)%3;
		if(format_index == 0)
			touch_conf(MDB_RETIS_TOUCH_CHANNEL,SET_POLLING_TAG,0);
			touch_conf(MDB_RETIS_TOUCH_CHANNEL,SET_RAW_TAG,0);
		else if(format_index == 1)
			touch_conf(MDB_RETIS_TOUCH_CHANNEL,SET_POLLING_TAG,0);
			touch_conf(MDB_RETIS_TOUCH_CHANNEL,SET_UNSIGNED_TAG,0);
		else if(format_index == 2)
			touch_conf(MDB_RETIS_TOUCH_CHANNEL,SET_POLLING_TAG,0);
			touch_conf(MDB_RETIS_TOUCH_CHANNEL,SET_SIGNED_TAG,0);
		else if(format_index == 3)
			touch_conf(MDB_RETIS_TOUCH_CHANNEL,SET_WAIT_TAG,0);
			touch_conf(MDB_RETIS_TOUCH_CHANNEL,SET_RAW_TAG,0);
		else if(format_index == 4)
			touch_conf(MDB_RETIS_TOUCH_CHANNEL,SET_WAIT_TAG,0);
			touch_conf(MDB_RETIS_TOUCH_CHANNEL,SET_UNSIGNED_TAG,0);
		else if(format_index == 5)
			touch_conf(MDB_RETIS_TOUCH_CHANNEL,SET_WAIT_TAG,0);
			touch_conf(MDB_RETIS_TOUCH_CHANNEL,SET_SIGNED_TAG,0);
	}
}

/******************************************************************************/
/*                                   MAIN                                     */
/******************************************************************************/
int main(void)
{	
	/* Clock setup for 40 MHz */
	CLKDIVbits.DOZEN   = 0;
	CLKDIVbits.PLLPRE  = 0;
	CLKDIVbits.PLLPOST = 0;
	PLLFBDbits.PLLDIV  = 78;
	while (OSCCONbits.LOCK != 1) ;

	/* Init basic components */	
	timer_set();		
	EE_leds_init();
	EE_daughter_leds_init();
	EE_switch_init();
	EE_buttons_init(NULL,0xFF);

	/* Application Init */
	my_console_1 = console_serial_config(MY_FIRST_SERIAL, 115200,
						CONSOLE_SERIAL_FLAG_BIT8_NO |
						CONSOLE_SERIAL_FLAG_BIT_STOP1);
	console_init(MY_FIRST_CONSOLE, my_console_1);
	if (console_open(MY_FIRST_CONSOLE) == -1)
		EE_led_on();
	else
		EE_led_off();

	tcd.x_range = MDB_RETIS_TOUCH_X_RANGE;
	tcd.y_range = MDB_RETIS_TOUCH_Y_RANGE;
	tcd.autostart = 1;

	//tcd.read_mode = [POLLING|BLOCKING]|[RAW_FORMAT|SIGNED_FORMAT|UNSIGNED_FORMAT];

	if(!EE_switch_1_get()) {
		if(!EE_switch_2_get() && !EE_switch_3_get())
			tcd.read_mode = POLLING|RAW_FORMAT;
		else if(!EE_switch_2_get() && EE_switch_3_get())
			tcd.read_mode = POLLING|UNSIGNED_FORMAT;
		else if(EE_switch_2_get() && !EE_switch_3_get())
			tcd.read_mode = POLLING|SIGNED_FORMAT;
	} else {
		if(!EE_switch_2_get() && !EE_switch_3_get())
			tcd.read_mode = WAIT|RAW_FORMAT;
		else if(!EE_switch_2_get() && EE_switch_3_get())
			tcd.read_mode = WAIT|UNSIGNED_FORMAT;
		else if(EE_switch_2_get() && !EE_switch_3_get())
			tcd.read_mode = WAIT|SIGNED_FORMAT;		
	}
	
	format_index = EE_switch_1_get()*3 + EE_switch_1_get()*2 + 
			EE_switch_3_get();

	touch_init(MDB_RETIS_TOUCH_CHANNEL, &tcd);

	SetRelAlarm(Alarm_Send, 200, 200);

	// Forever loop: background activities (if any) should go here
	for (;;);
}
/******************************************************************************/
/*                                     ISRs                                   */
/******************************************************************************/
ISR2(_T1Interrupt)
{
	IFS0bits.T1IF = 0;
	CounterTick(TASK_COUNTER);
}

