<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.1"
                xmlns:retis_xslt='retis_xslt_python/retis_xslt' xsl:exclude-result-prefixes='retis_xslt'>
    <xsl:output method="text" encoding="ASCII"/>  
    <xsl:key name="devices" match="blockset/*" use="name()"/>
    <xsl:template match="blockset[@board='flex_motionboard' and @mcu='dspic33FJ256MC710']">
        <xsl:variable name="base_dir" select="/retis/config/output_dir/@path"/>
        <xsl:variable name="file_name" select="/retis/config/lib_retis_inc/@name"/>
        <xsl:variable name="output" select="retis_xslt:register_output_file(concat($base_dir, '/', $file_name))"/>
        <xsl:document method="text" href="{$output}">
#ifndef __retis_library_include_h__
#define __retis_library_include_h__

/* ************************************************************************** */
/*                     Include RETIS libray components                        */
/* ************************************************************************** */
            <xsl:for-each select="*[generate-id() = generate-id(key('devices',name())[1])]">
#include "retis_<xsl:value-of select="name()"/>.h"
            </xsl:for-each>

/* ************************************************************************** */
/*                        Functional Names Mapping                            */
/* ************************************************************************** */
            <xsl:for-each select="*">
#define         <xsl:value-of select="@functional_name"/> 
                <xsl:text> </xsl:text> 
                <xsl:value-of select="@hw_id"/>
            </xsl:for-each>

#endif /* __retis_library_include_h__ */
        </xsl:document>
    </xsl:template>

    <!-- NOTE: This template is called with xsl:apply-templates by the erika.xsl to create 
               the APP_SRC entries in the CPU_DATA section of the OIL file.
               This template make use of a named-templated 'add_lib_retis_sources' defined by 
               the erika.xsl.
     -->
    <xsl:template match="blockset[@board='flex_motionboard' and @mcu='dspic33FJ256MC710']" mode="build_oil_app_src">
        <xsl:variable name="arch">dspic_flex_motionboard</xsl:variable>
        <xsl:call-template name="add_lib_retis_sources"> 
	    	<xsl:with-param name="arch_name" select="$arch"/>
        </xsl:call-template>
    </xsl:template>

    <!-- NOTE: This template is called with xsl:apply-templates by the erika.xsl to create 
               the LIB section of the OIL file.
     -->
    <xsl:template match="blockset[@board='flex_motionboard']/touch" mode="build_oil_lib_data">
        EE_OPT = "__ADD_LIBS__";
		EE_OPT = "__USE_TOUCH__";
        LIB = ENABLE {
            NAME = "TOUCH";
        };
    </xsl:template>

    <!-- NOTE: This template is called with xsl:apply-templates by the erika.xsl to create 
               the LIB section of the OIL file.
     -->
    <xsl:template match="blockset[@board='flex_motionboard']/touch" mode="build_oil_tasks_extra">
    TASK TASK_TOUCH_MANAGER {
       PRIORITY = 3;
       STACK = SHARED;
       SCHEDULE = FULL;
    };
    </xsl:template>

    <!-- NOTE: This template is called with xsl:apply-templates by the erika.xsl to create 
               the BOARD section of the OIL file.
     -->
   <xsl:template match="blockset[@board='flex_motionboard']" mode="build_oil_board_data">
        BOARD_DATA = EE_FLEX {
            USELEDS = TRUE;
        };
        EE_OPT = "__USE_MOTIONBOARD__";
        EE_OPT = "__USE_LEDS__";
        EE_OPT = "__USE_SWITCH__";
    </xsl:template>

</xsl:stylesheet>
