from PyQt4 import QtGui, QtCore 
from tree import xmlTree
from readXML import read
import sys

class MainWindow(QtGui.QMainWindow):
	def __init__(self, hwDict, hclist, argv):
		QtGui.QMainWindow.__init__(self) 
		
		self.hwDict = hwDict
		self.label = dict()
		self.hw = dict()
		self.sub = dict()
		self.block = dict()
		self.led = []
		self.servomotor =[]
		self.lcd2x16 =[]
		self.touch= []
		self.button = []
		self.init_window()
		self.t = xmlTree()
		self.l = hclist
		self.argv = argv

	def init_window(self):                               
		self.setWindowTitle('Retis Lab')
		self.statusBar().showMessage('Tool for device mapping') 
		
		global cWidget 
		cWidget = QtGui.QWidget(self)
		general = self.create_layout('horizontal', 5)
		
		fill_part = self.create_layout('vertical', 5)
		button_part = self.create_layout('vertical', 5)

		open_butt = self.create_button("Open matlab file",'clicked()', 0, button_part)
		open_xml_butt = self.create_button("Open xml file",'clicked()', 0, button_part)
		open_gen_butt = self.create_button("Generate code",'clicked()', self.create_blockset_list, button_part)
		open_quit_butt = self.create_button("Quit",'clicked()', QtCore.SLOT('close()'), button_part)

		dummy = []
		t_part = self.create_layout('horizontal', 5)
		dummy.append(self.create_line('sub', 'Subsystem', t_part, True))
		dummy.append(self.create_line('sub_name', 'Sub1', t_part, False))
		dummy.append(self.create_line('sub_order', '1', t_part, False))	
		self.sub['sub_1'] = dummy
		del dummy
		fill_part.addLayout(t_part)	
		
		hw_part = self.create_layout('horizontal', 5)
	
		# The first MCU is set by default
		self.create_label('mcu', "MCU:", hw_part)
		self.hw['mcu'] = self.create_combo('mcu', self.hwDict.keys(), hw_part)
		self.selected_mcu = self.hwDict.keys()[0]
		# Add the call-back for changing mcu handling		

		self.create_label('board', "Board:", hw_part)
		boards =[]
		for e in self.hwDict[self.hwDict.keys()[0]]:
			boards.append(e)
#		print boards
		self.hw['board'] = self.create_combo('board', boards, hw_part)
		self.connect(self.hw['board'], QtCore.SIGNAL("activated(int)"), self.changeBoard)
		self.changeBoard(0)
		self.selected_board = boards[0]
		del boards
		self.create_tlist()
		
		fill_part.addLayout(hw_part)
		blockset = self.create_layout('vertical', 5)

#		print self.hwDict
	
		block_dict = dict()
		block_part = []
		
		eList = self.fill_elist()
		for e in eList:
#			print e[0]
			bp = self.create_layout('horizontal', 5)
			if e[0] in block_dict:
				block_dict[e[0]] += 1
			else:
				block_dict[e[0]] = 0
#			print block_dict
			name_ = e[0] + '_' + str(block_dict[e[0]])
#			print 'name_ ', name_
			dummy = []
			dummy.append(self.create_line(e[0] + '_name' + str(block_dict[e[0]]), e[0], bp, True))	
			dummy.append(self.create_line(e[0] + '_func' + str(block_dict[e[0]]), e[1], bp, True))
#			print e[0], self.hwDict[self.selected_mcu][self.selected_board][e[0]]
			dummy.append(self.create_combo(e[0] + '_id' + str(block_dict[e[0]]), self.hwDict[self.selected_mcu][self.selected_board][e[0]], bp))
			block_part.append(bp)
			self.block[name_] = dummy
			del dummy
		
		if block_part:
			for e in block_part:
				blockset.addLayout(e)

		fill_part.addLayout(blockset)

		general.addLayout(fill_part)
		general.addLayout(button_part)

		
		cWidget.setLayout(general)
		self.setCentralWidget(cWidget)

	def create_button(self, name,_signal, _slot, _layout):
		button = QtGui.QPushButton(name, cWidget)
		button.setFont(QtGui.QFont("Times", 10, QtGui.QFont.Bold))
		_layout.addWidget(button)
		if _slot:
#			print _slot
			self.connect(button, QtCore.SIGNAL(_signal), _slot)
		return button

	def create_layout(self, _type, spacing):
		values = {'horizontal' 	: self.horizontal(spacing), 
			  'vertical' 	: self.vertical(spacing),
			  'grid'	: self.grid(spacing)}
		print _type
         	return values[_type]

	def horizontal(self, spacing):
		hBox = QtGui.QHBoxLayout()
		hBox.setSpacing(spacing)
		return hBox

	def vertical(self, spacing):
		vBox = QtGui.QVBoxLayout()
		vBox.setSpacing(spacing)
		return vBox

	def grid(self, spacing):
		grid = QtGui.QGridLayout()
		grid.setSpacing(spacing)
		return grid

	def create_label(self, name, string, layout):
		_label = QtGui.QLabel(string, cWidget)
		self.label[name] = _label
		layout.addWidget(self.label[name])
	
	def create_line(self, name, string, layout, read):
		_line = QtGui.QLineEdit(cWidget)
		_line.setText(string)
		_line.setReadOnly(read)
		layout.addWidget(_line)
		return _line

	def create_combo(self, name, items, layout):
		_combo = QtGui.QComboBox(cWidget)
		self.fill_items(_combo, items)
		_combo.setCurrentIndex(0)
		layout.addWidget(_combo)
		return _combo

	def fill_items(self, _combo, items):
		if items:
			for e in items:
				_combo.addItem(str(e))

	def changeBoard(self, index):
		self.selected_board = str(self.hw['board'].itemText(index))
		for e in self.block:
			print 'e', e
			for i in self.hwDict[self.selected_mcu][self.selected_board]:
				print 'i', i
				if i in e:
					self.block[e][2].clear()
					print 'self.hwDict[self.selected_mcu][self.selected_board][i[0]] ', self.hwDict[self.selected_mcu][self.selected_board][i]
					self.fill_items(self.block[e][2], self.hwDict[self.selected_mcu][self.selected_board][i])
	
	def create_blockset_attribute(self):
		_list = []
		_list.append(['board', str(self.hw['board'].itemText(self.hw['board'].currentIndex()))]) 
		_list.append(['mcu', str(self.hw['mcu'].itemText(self.hw['mcu'].currentIndex()))])
		return _list

	def create_blockset_children(self):
		_block = []
		for e in self.block:
			if 'led' in e:
				_list = []
				_list.append(['functional_name', str(self.block[e][1].text())])
				_list.append(['hw_id', str(self.block[e][2].itemText(self.block[e][2].currentIndex()))])
				_block.append(['led', _list, None])
			elif 'button' in e:
				_list = []
				_list.append(['functional_name', str(self.block[e][1].text())])
				_list.append(['hw_id', str(self.block[e][2].itemText(self.block[e][2].currentIndex()))])
				_block.append(['button', _list, None])

			elif 'servomotor' in e:
				_list = []
				_list.append(['functional_name', str(self.block[e][1].text())])
				_list.append(['hw_id', str(self.block[e][2].itemText(self.block[e][2].currentIndex()))])
				_block.append(['servomotor', _list, None])
			elif 'lcd2x16' in e:
				_list = []
				_list.append(['functional_name', str(self.block[e][1].text())])
				_list.append(['hw_id', str(self.block[e][2].itemText(self.block[e][2].currentIndex()))])
				_block.append(['lcd2x16', _list, None])
			elif 'touch' in e:
				_list = []
				_list.append(['functional_name', str(self.block[e][1].text())])
				_list.append(['hw_id', str(self.block[e][2].itemText(self.block[e][2].currentIndex()))])
				_block.append(['touch', _list, None])
		return _block	

	def create_blockset_list(self):
                ## GENERATE XML FILE
		_list = []
		self.t.create_root('retis', None)
		_list.append(['blockset', self.create_blockset_attribute(), self.create_blockset_children()])
		self.t.create_subtree('root', self.l)
		self.t.create_subtree('root', self.create_tlist())
		self.t.create_subtree('root', _list)
#		self.t.print_to_screen()
		self.t.write_to_file()
                ## CALL ERIKA CODE GENERATOR
		sys.path.append("..")
		try:
			#TODO Switch on target OS type
			import erika_codegen
                        erika_codegen.do_infinite_wait = False
			erika_codegen.main(self.argv)
		except:
			print "ERROR: Unable to call erika code generator"
			print sys.exc_info()[1]
			print ""

	def fill_elist(self):
		r = read() 
		bList = []
		for e in self.hwDict[self.selected_mcu][self.selected_board]:
#			print 'e', e
			reflist = r.search_by_name(e)
			if len(reflist):
				for i in reflist:
					eList = [e]
					for u in i.attributes.keys():
						eList.append(i.attributes[u].value.encode('ascii'))
	#					print i.attributes[u].name
	#					print i.attributes[u].value
					bList.append(eList)
		return bList


	def create_tlist(self):
		_name = str(self.sub['sub_1'][1].text())
		_order = str(self.sub['sub_1'][2].text())
		return [['os_mapping', [['os_name', 'erika']], [['task', [['task_name', "FIRST_TASK"], ['period', "50"], ['priority', '1']], [['subsys', [['subsys_name', _name], ['subsys_order', _order]], None]], None]]]]
	
