<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.1"
                xmlns:retis_xslt='retis_xslt_python/retis_xslt' xsl:exclude-result-prefixes='retis_xslt'>
    <xsl:output method="text" encoding="ASCII"/>  
    <xsl:key name="devices" match="blockset/*" use="name()"/>
    <xsl:template match="os_mapping[@os_name='erika']">
        <xsl:variable name="base_dir" select="/retis/config/output_dir/@path"/>
        <xsl:variable name="main_filename">main.c</xsl:variable>
        <xsl:variable name="model_c_filename">retis_model.c</xsl:variable>
        <xsl:variable name="model_h_filename">retis_model.h</xsl:variable>
        <xsl:variable name="output_main" select="retis_xslt:register_output_file(concat($base_dir, '/', $main_filename))"/>
        <xsl:variable name="output_model_c" select="retis_xslt:register_output_file(concat($base_dir, '/', $model_c_filename))"/>
        <xsl:variable name="output_model_h" select="retis_xslt:register_output_file(concat($base_dir, '/', $model_h_filename))"/>
        <xsl:variable name="output_oil" select="retis_xslt:register_output_file(concat($base_dir, '/conf.oil'))"/>
<!-- ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                            retis_model.h            
     ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ -->
        <xsl:document method="text" href="{$output_model_h}">
#ifndef __retis_model_h__
#define __retis_model_h__

void retis_model_library_init(void);
void retis_model_blockset_init(void); 
void retis_model_tasks_start(void); 

#endif /* __retis_model_h__ */

        </xsl:document>
<!-- ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                            retis_model.c            
     ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ -->
        <xsl:document method="text" href="{$output_model_c}">		
#include "ee.h"
		<xsl:for-each select="task/subsys">
#include "<xsl:value-of select="@subsys_name"/>.h"
        </xsl:for-each> 

            <xsl:for-each select="task">
#define TASK_<xsl:value-of select="@task_name"/>_PERIOD <xsl:value-of select="@period"/> 
            </xsl:for-each>

/* ************************************************************************** */
/*                            TASK Definitions                                */
/* ************************************************************************** */
            <xsl:for-each select="task">
TASK(<xsl:value-of select="@task_name"/>) 
{
                <xsl:for-each select="subsys">
        <xsl:value-of select="@subsys_name"/>_step();
                </xsl:for-each>
}
            </xsl:for-each>

/* ************************************************************************** */
/*                        Initialization Functions                            */
/* ************************************************************************** */
void retis_model_library_init(void)
{
           <xsl:for-each select="/retis/blockset/*[generate-id() = generate-id(key('devices',name())[1])]">
        /* retis_<xsl:value-of select="name()"/>_some_global_API_action(); */
           </xsl:for-each>
}

void retis_model_blockset_init(void) 
{
            <xsl:for-each select="task/subsys">
        <xsl:value-of select="@subsys_name"/>_initialize();
            </xsl:for-each>
}

void retis_model_tasks_start(void) 
{
            <xsl:for-each select="task">
        SetRelAlarm(ALARM_<xsl:value-of select="@task_name"/>, 1000, TASK_<xsl:value-of select="@task_name"/>_PERIOD);
            </xsl:for-each>
}

        </xsl:document>
<!-- ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                            main.c            
     ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ -->
        <xsl:document method="text" href="{$output_main}">
#include "ee.h" 
#include "cpu/pic30/inc/ee_irqstub.h" // TODO: REMOVE IN CODE GENERATOR!
#include "retis_model.h" 

/* ************************************************************************** */
/*                              Fuse Bits                                     */
/* ************************************************************************** */
_FWDT(FWDTEN_OFF);      /* Watchdog Timer Enabled/disabled by user software */
_FGS(GCP_OFF);          /* Disable Code Protection */
_FOSCSEL(FNOSC_PRIPLL); /* Primary (XT, HS, EC) Oscillator without PLL */
_FOSC(OSCIOFNC_ON &#38; POSCMD_XT); /* OSC2 Pin Function: OSC2 is Clock Output
                                 * Primary Oscillator Mode: XT Crystanl */

/* ************************************************************************** */
/*                            System Timer                                    */
/* ************************************************************************** */
static void hw_timer_init(void)
{
        /* TIMER 1  */
        T1CON = 0;          /* Stops the Timer1 and reset control reg */
        TMR1 = 0;           /* Clear contents of the timer register */
        PR1 = 0x9C40;       /* Load the Period register -> 1ms @ 40MHz */
        IPC0bits.T1IP = 5;  /* Set Timer1 priority to 5 */
        IFS0bits.T1IF = 0;  /* Clear the Timer1 interrupt status flag */
        IEC0bits.T1IE = 1;  /* Enable Timer1 interrupts */
        T1CONbits.TON = 1;  /* Start Timer1 without prescaler */
}

ISR2(_T1Interrupt)
{
	/* ISR of type 2 for system timer */
        IFS0bits.T1IF = 0;
        CounterTick(RETIS_MODEL_TASK_COUNTER);
}

/* ************************************************************************** */
/*                                 MAIN                                       */
/* ************************************************************************** */
int main(void)
{
        /* Clock setup for 40MHz */
        CLKDIVbits.DOZEN   = 0;
        CLKDIVbits.PLLPRE  = 0;
        CLKDIVbits.PLLPOST = 0;
        PLLFBDbits.PLLDIV  = 78;
        while(OSCCONbits.LOCK != 1) ;

	/* Initialize system */
        hw_timer_init();
        retis_model_library_init();
        retis_model_blockset_init();

	/* Start periodic tasks */
        retis_model_tasks_start();

        for (;;) ;  /* Background activity here */

        return 0;
}

        </xsl:document>
<!-- ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                            conf.oil            
     ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ -->
        <xsl:document method="text" href="{$output_oil}">
            <xsl:variable name="base" select="/retis/config/lib_retis_dir/@path"/>
CPU mySystem {

    OS myOs {

            <xsl:variable name="host_os" select="/retis/config/host_machine/@os_name"/>
            <xsl:choose>
                <xsl:when test="$host_os='linux'">
        CFLAGS = "-I<xsl:value-of select="$base"/>/inc";
                </xsl:when>
                <xsl:when test="$host_os='win32'">
        CFLAGS = "-I'<xsl:value-of select="$base"/>\inc'";
                </xsl:when>
            </xsl:choose>

        EE_OPT = "DEBUG";
            <xsl:apply-templates select="/retis/blockset" mode="build_oil_cpu_data"/> 
            /* Main and tasks */

            APP_SRC = "<xsl:value-of select="$main_filename"/>";

            APP_SRC = "<xsl:value-of select="$model_c_filename"/>";

            /* Generated Subsystems */
            <xsl:for-each select="*/subsys">
            APP_SRC = "<xsl:value-of select="@subsys_name"/>.c";
            </xsl:for-each>
            /* RETIS library sources */
            <xsl:apply-templates select="/retis/blockset" mode="build_oil_app_src"/> 
        };
            <xsl:apply-templates select="/retis/blockset" mode="build_oil_mcu_data"/> 
            <xsl:apply-templates select="/retis/blockset" mode="build_oil_board_data"/> 
            <xsl:apply-templates select="/retis/blockset" mode="build_oil_lib_data"/> 
        KERNEL_TYPE = FP;
    };

    COUNTER RETIS_MODEL_TASK_COUNTER;    

    /* TASK Declarations */
            <xsl:for-each select="task">
    TASK <xsl:value-of select="@task_name"/> {
        PRIORITY = 1;
        STACK = SHARED;
        SCHEDULE = FULL;
    };
    ALARM ALARM_<xsl:value-of select="@task_name"/> {
        COUNTER = "RETIS_MODEL_TASK_COUNTER";
        ACTION = ACTIVATETASK { TASK =  "<xsl:value-of select="@task_name"/>"; };
    };
            </xsl:for-each>
            <xsl:apply-templates select="/retis/blockset" mode="build_oil_tasks_extra"/> 
};
        </xsl:document>
    </xsl:template>

    <xsl:template name="add_lib_retis_sources">
        <xsl:param name="arch_name"/>
        <xsl:variable name="base" select="/retis/config/lib_retis_dir/@path"/>
        <xsl:variable name="host_os" select="/retis/config/host_machine/@os_name"/>
        <xsl:choose>
            <xsl:when test="$host_os='linux'">
                <xsl:for-each select="/retis/blockset/*[generate-id() = generate-id(key('devices',name())[1])]">
            APP_SRC = "<xsl:value-of select="$base"/>/arch/<xsl:value-of select="$arch_name"/>/retis_<xsl:value-of select="name()"/>.c";
                </xsl:for-each>
            </xsl:when>
            <xsl:when test="$host_os='win32'">
                <xsl:for-each select="/retis/blockset/*[generate-id() = generate-id(key('devices',name())[1])]">
            APP_SRC = "$(shell cygpath -u '<xsl:value-of select="$base"/>')/arch/<xsl:value-of select="$arch_name"/>/retis_<xsl:value-of select="name()"/>.c";
                </xsl:for-each>
            </xsl:when>
        </xsl:choose>
    </xsl:template>
</xsl:stylesheet>
