#! /usr/bin/env python

import sys
import os
from optparse import OptionParser

def get_codegen_default_path():
    if os.name == "posix":
        retis_codegen_path = "/default_path/to/mbd_retis/retis_code_generator"
    elif os.name == "nt":
        retis_codegen_path = "Z:\\retis_code_generator"
    else:
        sys.exit("OS not supported")
    return retis_codegen_path

def parse_option(argv):
    parser = OptionParser("usage: %prog [options]")
    parser.add_option("-i", "--input-xml", 
                      action="store", type="string", dest="xml_fname",
                      help="Input XML file name", metavar="xml_file")
    parser.add_option("-r", "--rules-xsl",
                      action="store", type="string", dest="xsl_fname",
                      help="XSL file name for XSLT rules", metavar="xsl_file")
    parser.add_option("-g", "--codegen-path",
                      action="store", type="string", dest="codegen_path",
                      help="Look for the code-generator in this directory", 
                      metavar="path")
    parser.add_option("-v", "--verbose",
                      action="store_true", dest="verbose", default=False,
                      help="Print status messages to stdout")
    (options, args) = parser.parse_args(argv)
    return (options.xml_fname, options.xsl_fname, 
            options.codegen_path, options.verbose)

def remove_double_blank_lines(filename):
    f = open(filename, 'r')
    src = f.readlines()
    f.close()
    out = ""
    s_prev = src[0].strip()
    for s in src:
        if s.strip() != '' or s_prev != '':
            out += s
        s_prev = s.strip()
    f = open(filename, 'w')
    f.write(out)
    f.close()

def main(argument=None):
    (xml_fname, xsl_fname, codegen_path, verbose) = parse_option(argument)
    argv = []
    if not xml_fname:
        argv.append("retis_mappings.xml")
    else:
        argv.append(xml_fname)
    if not codegen_path:
        codegen_path = get_codegen_default_path()
    if verbose:
        argv.append("-v")
    if not xsl_fname:
        argv.append(os.path.join(codegen_path, 
                    "retis_transformation_rules.xsl"))
    else:
        argv.append(xsl_fname)
    sys.path.append(codegen_path)
    print ""
    print "*********** RETIS Code Generator *************"
    try:
        import retis_xslt
    except:
        print "Exception importing retis_xslt:"
        print sys.exc_info()[1]
        print "FAILURE"
        print "**********************************************"
        print ""
        return False
    try:
        retis_xslt.main(argv)
        for f in retis_xslt.output_file_list:
            remove_double_blank_lines(f)
    except:
        print "Exception executing retis_xslt:"
        print sys.exc_info()[1]
        print "FAILURE"
        print "**********************************************"
        print ""
        return False
    print "SUCCESS"
    print "**********************************************"
    print ""
    return True

# MAIN #

if __name__ == '__main__':
    main(sys.argv[1:])
