#! /usr/bin/env python

import libxml2
import libxslt
import sys
import os
from optparse import OptionParser

is_verbose = False
output_file_list = []

def parse_option(argv):
    parser = OptionParser("usage: %prog [options] xml xsl")
    parser.add_option("-o", "--output-file", 
                      action="store", type="string", dest="out_filename",
                      help="Output file name.", metavar="output_file")
    parser.add_option("-v", "--verbose",
                      action="store_true", dest="verbose", default=False,
                      help="Print status messages to stdout")
    (options, args) = parser.parse_args(argv)
    if len(args) < 2:
        sys.exit("Too few arguments, see help.")
    in_xml = args[0];
    in_xsl = args[1];
    if not options.out_filename:
        out_file = ""
    else:
        out_file = options.out_filename
    global is_verbose
    is_verbose = options.verbose
    return (in_xml, in_xsl, out_file)

def add_out_files(ctx, str):
    output_file_list.append(str)
    return str

def perfom_xslt(xml, xsl, out):
    global output_file_list
    output_file_list = []
    libxslt.registerExtModuleFunction("register_output_file", 
                                      "retis_xslt_python/retis_xslt", 
                                      add_out_files)
    styledoc = libxml2.parseFile(xsl)
    style = libxslt.parseStylesheetDoc(styledoc)
    doc = libxml2.parseFile(xml)
    result = style.applyStylesheet(doc, None)
    if  out != "":
        style.saveResultToFilename(out, result, 0)
    style.freeStylesheet()
    doc.freeDoc()
    result.freeDoc()

def main(argv=None):
    global is_verbose
    global output_file_list
    if argv is None:
        argv = sys.argv[1:]
    (src_xml, src_xsl, dst_file) = parse_option(argv)
    if is_verbose:
        print "Start XML Translation"
    if not os.access(src_xml, os.R_OK):
        sys.exit("Cannot access XML file: " + src_xml)
    if not os.access(src_xsl, os.R_OK):
        sys.exit("Cannot access XSL file: " + src_xsl)
    if is_verbose:
        print "- Input XML file: " + src_xml
        print "- Input XSL file: " + src_xsl
    perfom_xslt(src_xml, src_xsl, dst_file)
    if is_verbose:
        print "- Output files: "
        if dst_file != "":
            output_file_list.append(dst_file)
        for f in output_file_list:
            print "  + ", f
    print "End of XML Translation"

# MAIN #

if __name__ == '__main__':
    main()
