#! /usr/bin/env python

import sys
import os
from optparse import OptionParser

def get_codegen_default_path():
    if os.name == "posix":
        retis_codegen_path = "/default_path/to/mbd_retis/retis_code_generator"
    elif os.name == "nt":
        retis_codegen_path = "C:\\MBD\\retis_code_generator"
    else:
        sys.exit("OS not supported")
    return retis_codegen_path

def parse_option(argv):
    parser = OptionParser("usage: %prog [options]")
    parser.add_option("-i", "--input-xml", 
                      action="store", type="string", dest="xml_fname",
                      help="Input XML file name", metavar="xml_file")
    parser.add_option("-p", "--codegen-path",
                      action="store", type="string", dest="codegen_path",
                      help="Look for the code-generator in this directory", 
                      metavar="path")
    parser.add_option("-v", "--verbose",
                      action="store_true", dest="verbose", default=False,
                      help="Print status messages to stdout")
    (options, args) = parser.parse_args(argv)
    return (options.xml_fname, options.codegen_path, options.verbose)

def main(argument=None):
    if not argument:
        argument = sys.argv[1:]
    (xml_fname, codegen_path, verbose) = parse_option(argument)
    argv = []
    if xml_fname:
        argv.append("-i")
        argv.append(xml_fname)
    if not codegen_path:
        codegen_path = get_codegen_default_path()
    argv.append("-g")
    argv.append(codegen_path)
    #if verbose:
    #    argv.append("-v")
    argv.append("-v")
    sys.path.append(codegen_path)
    sys.path.append(os.path.join(codegen_path, "gui"))
    print ""
    try:
        #import erika_codegen
        #erika_codegen.main(argv)
        import mapping
        mapping.main(argv)
    except:
        print "ERROR: ",
        print "Exception while executing code generator." 
        print sys.exc_info()[1]
        print ""
        return False
    print ""
    return True

# MAIN #

if __name__ == '__main__':
    main()
